@@ -1,10 +1,12 @@
 import argparse
+import json
 import glob
 import os
 import re
 import sys
 import types
 from collections import OrderedDict
+import textwrap
 
 if sys.version_info >= (3, 0):
     from io import StringIO
@@ -147,31 +149,32 @@ class DefaultConfigFileParser(ConfigFileParser):
             line = line.strip()
             if not line or line[0] in ["#", ";", "["] or line.startswith("---"):
                 continue
-            white_space = "\\s*"
-            key = r"(?P<key>[^:=;#\s]+?)"
-            value = white_space+r"[:=\s]"+white_space+"(?P<value>.+?)"
-            comment = white_space+"(?P<comment>\\s[;#].*)?"
-
-            key_only_match = re.match("^" + key + comment + "$", line)
-            if key_only_match:
-                key = key_only_match.group("key")
-                items[key] = "true"
-                continue
-
-            key_value_match = re.match("^"+key+value+comment+"$", line)
-            if key_value_match:
-                key = key_value_match.group("key")
-                value = key_value_match.group("value")
 
+            match = re.match(r'^(?P<key>[^:=;#\s]+)\s*'
+                             r'(?:(?P<equal>[:=\s])\s*([\'"]?)(?P<value>.+?)?\3)?'
+                             r'\s*(?:\s[;#]\s*(?P<comment>.*?)\s*)?$', line)
+            if match:
+                key = match.group("key")
+                equal = match.group('equal')
+                value = match.group("value")
+                comment = match.group("comment")
+                if value is None and equal is not None and equal != ' ':
+                    value = ''
+                elif value is None:
+                    value = "true"
                 if value.startswith("[") and value.endswith("]"):
-                    # handle special case of lists
-                    value = [elem.strip() for elem in value[1:-1].split(",")]
-
+                    # handle special case of k=[1,2,3] or other json-like syntax
+                    try:
+                        value = json.loads(value)
+                    except Exception as e:
+                        # for backward compatibility with legacy format (eg. where config value is [a, b, c] instead of proper json ["a", "b", "c"]
+                        value = [elem.strip() for elem in value[1:-1].split(",")]
+                if comment:
+                    comment = comment.strip()[1:].strip()
                 items[key] = value
-                continue
-
-            raise ConfigFileParserException("Unexpected line {} in {}: {}".format(i,
-                getattr(stream, 'name', 'stream'), line))
+            else:
+                raise ConfigFileParserException("Unexpected line {} in {}: {}".format(i,
+                    getattr(stream, 'name', 'stream'), line))
         return items
 
     def serialize(self, items):
@@ -193,7 +196,7 @@ class ConfigparserConfigFileParser(ConfigFileParser):
     def get_syntax_description(self):
         msg = """Uses configparser module to parse an INI file which allows multi-line
         values.
-        
+
         Allowed syntax is that for a ConfigParser with the following options:
 
             allow_no_value = False,
@@ -202,7 +205,7 @@ class ConfigparserConfigFileParser(ConfigFileParser):
             empty_lines_in_values = False
 
         See https://docs.python.org/3/library/configparser.html for details.
-        
+
         Note: INI file sections names are still treated as comments.
         """
         return msg
@@ -224,6 +227,7 @@ class ConfigparserConfigFileParser(ConfigFileParser):
             config.read_string(stream.read())
         except Exception as e:
             raise ConfigFileParserException("Couldn't parse config file: %s" % e)
+
         # convert to dict and remove INI section names
         result = OrderedDict()
         for section in config.sections():
@@ -260,6 +264,7 @@ class ConfigparserConfigFileParser(ConfigFileParser):
         stream.seek(0)
         return stream.read()
 
+
 class YAMLConfigFileParser(ConfigFileParser):
     """Parses YAML config files. Depends on the PyYAML module.
     https://pypi.python.org/pypi/PyYAML
@@ -271,7 +276,7 @@ class YAMLConfigFileParser(ConfigFileParser):
         return msg
 
     def _load_yaml(self):
-        """lazy-import PyYAML so that configargparse doesn't have to dependend
+        """lazy-import PyYAML so that configargparse doesn't have to depend
         on it unless this parser is used."""
         try:
             import yaml
@@ -300,12 +305,13 @@ class YAMLConfigFileParser(ConfigFileParser):
         for key, value in parsed_obj.items():
             if isinstance(value, list):
                 result[key] = value
+            elif value is None:
+                pass
             else:
                 result[key] = str(value)
 
         return result
 
-
     def serialize(self, items, default_flow_style=False):
         """Does the inverse of config parsing by taking parsed values and
         converting them back to a string representing config file contents.
@@ -352,11 +358,11 @@ class ArgumentParser(argparse.ArgumentParser):
                 environment variable)
             default_config_files: When specified, this list of config files will
                 be parsed in order, with the values from each config file
-                taking precedence over pervious ones. This allows an application
+                taking precedence over previous ones. This allows an application
                 to look for config files in multiple standard locations such as
                 the install directory, home directory, and current directory.
                 Also, shell * syntax can be used to specify all conf files in a
-                directory. For exmaple:
+                directory. For example:
                 ["/etc/conf/app_config.ini",
                  "/etc/conf/conf-enabled/*.ini",
                 "~/.my_app_config.ini",
@@ -366,7 +372,7 @@ class ArgumentParser(argparse.ArgumentParser):
                 configargparse args will be ignored. If false, they will be
                 processed and appended to the commandline like other args, and
                 can be retrieved using parse_known_args() instead of parse_args()
-            config_file_open_func: function used to open a config file for reading 
+            config_file_open_func: function used to open a config file for reading
                 or writing. Needs to return a file-like object.
             config_file_parser_class: configargparse.ConfigFileParser subclass
                 which determines the config file format. configargparse comes
@@ -448,24 +454,34 @@ class ArgumentParser(argparse.ArgumentParser):
             config_file_contents: String. Used for testing.
             env_vars: Dictionary. Used for testing.
         """
-        args, argv = self.parse_known_args(args = args,
-            namespace = namespace,
-            config_file_contents = config_file_contents,
-            env_vars = env_vars)
+        args, argv = self.parse_known_args(
+            args=args,
+            namespace=namespace,
+            config_file_contents=config_file_contents,
+            env_vars=env_vars,
+            ignore_help_args=False)
+
         if argv:
             self.error('unrecognized arguments: %s' % ' '.join(argv))
         return args
 
-
-    def parse_known_args(self, args = None, namespace = None,
-                         config_file_contents = None, env_vars = os.environ):
+    def parse_known_args(
+            self,
+            args=None,
+            namespace=None,
+            config_file_contents=None,
+            env_vars=os.environ,
+            ignore_help_args=False,
+    ):
         """Supports all the same args as the ArgumentParser.parse_args(..),
         as well as the following additional args.
 
         Additional Args:
             args: a list of args as in argparse, or a string (eg. "-x -y bla")
-            config_file_contents: String. Used for testing.
-            env_vars: Dictionary. Used for testing.
+            config_file_contents (str). Used for testing.
+            env_vars (dict). Used for testing.
+            ignore_help_args (bool): This flag determines behavior when user specifies --help or -h. If False,
+                it will have the default behavior - printing help and exiting. If True, it won't do either.
         """
         if args is None:
             args = sys.argv[1:]
@@ -477,6 +493,9 @@ class ArgumentParser(argparse.ArgumentParser):
         for a in self._actions:
             a.is_positional_arg = not a.option_strings
 
+        if ignore_help_args:
+            args = [arg for arg in args if arg not in ("-h", "--help")]
+
         # maps a string describing the source (eg. env var) to a settings dict
         # to keep track of where values came from (used by print_values()).
         # The settings dicts for env vars and config files will then map
@@ -737,6 +756,11 @@ class ArgumentParser(argparse.ArgumentParser):
             elif value.lower() in ("false", "no", "0"):
                 # don't append when set to "false" / "no"
                 pass
+            elif isinstance(action, argparse._CountAction):
+                for arg in args:
+                    if any([arg.startswith(s) for s in action.option_strings]):
+                        value = 0
+                args += [action.option_strings[0]] * int(value)
             else:
                 self.error("Unexpected value for %s: '%s'. Expecting 'true', "
                            "'false', 'yes', 'no', '1' or '0'" % (key, value))
@@ -836,18 +860,33 @@ class ArgumentParser(argparse.ArgumentParser):
 
             if not user_config_file:
                 continue
-            # validate the user-provided config file path
-            user_config_file = os.path.expanduser(user_config_file)
-            if not os.path.isfile(user_config_file):
-                self.error('File not found: %s' % user_config_file)
 
-            config_files += [self._config_file_open_func(user_config_file)]
+            # open user-provided config file
+            user_config_file = os.path.expanduser(user_config_file)
+            try:
+                stream = self._config_file_open_func(user_config_file)
+            except Exception as e:
+                if len(e.args) == 2:  # OSError
+                    errno, msg = e.args
+                else:
+                    msg = str(e)
+                # close previously opened config files
+                for config_file in config_files:
+                    try:
+                        config_file.close()
+                    except Exception:
+                        pass
+                self.error("Unable to open config file: %s. Error: %s" % (
+                    user_config_file, msg
+                ))
+
+            config_files += [stream]
 
         return config_files
 
     def format_values(self):
         """Returns a string with all args and settings and where they came from
-        (eg. commandline, config file, enviroment variable or default)
+        (eg. commandline, config file, environment variable or default)
         """
         source_key_to_display_value_map = {
             _COMMAND_LINE_SOURCE_KEY: "Command Line Args: ",
@@ -929,10 +968,12 @@ class ArgumentParser(argparse.ArgumentParser):
             msg += (" If an arg is specified in more than one place, then "
                 "commandline values override %s.") % (
                 " which override ".join(value_sources))
-        if msg:
-            self.description = (self.description or "") + " " + msg
 
-        return argparse.ArgumentParser.format_help(self)
+        text_width = max(self._get_formatter()._width, 11)
+        msg = textwrap.fill(msg, text_width)
+
+        return (argparse.ArgumentParser.format_help(self)
+              + ("\n{}\n".format(msg) if msg != "" else ""))
 
 
 def add_argument(self, *args, **kwargs):
@@ -943,7 +984,7 @@ def add_argument(self, *args, **kwargs):
     Additional Args:
         env_var: If set, the value of this environment variable will override
             any config file or default values for this arg (but can itself
-            be overriden on the commandline). Also, if auto_env_var_prefix is
+            be overridden on the commandline). Also, if auto_env_var_prefix is
             set in the constructor, this env var name will be used instead of
             the automatic name.
         is_config_file_arg: If True, this arg is treated as a config file path
